﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace IGCheckboxTree
{
    /// <summary>
    /// Class defines logic for a Differences tree item (actual difference will have zero children)
    /// </summary>
    public class Item : NotificationObject
    {
        /// <summary>
        /// Ctor
        /// </summary>
        /// <param name="name">Name of the node/item</param>
        public Item(string name)
        {
            Name = name;
            this.Children = new ObservableCollection<Item>();
            Children.CollectionChanged += Children_CollectionChanged;
            this.IsChecked = true;
        }

        /// <summary>
        /// The item's name
        /// </summary>
        public string Name { get; private set; }

        /// <summary>
        /// The parent item of the item (gets set when Initialized)
        /// </summary>
        Item Parent;

        /// <summary>
        /// Items collection of children items
        /// </summary>
        public ObservableCollection<Item> Children { get; private set; }

        /// <summary>
        /// Bool for if item is Initially Selected
        /// </summary>
        public bool IsInitiallySelected { get; set; }

        private bool _isChecked;
        /// <summary>
        /// Bool for whether the Item is checked (true) or not (false) 
        /// </summary>
        public bool IsChecked
        {
            get { return _isChecked; }
            set
            {
                if (_isChecked != value)
                {
                    _isChecked = value;
                    FirePropertyChanged(() => IsChecked);
                    FirePropertyChanged(() => Accepted);
                }
            }
        }

        /// <summary>
        /// Number of accepted differences in the items collection (or itself if no children)
        /// </summary>
        public int Accepted
        {
            get { return CountTotalAccepted(this); }
        }

        /// <summary>
        /// Total number of differences in the items collection (or itself if no children)
        /// </summary>
        public int Differences
        {
            get { return CountTotalDifferences(this); }
        }

        /// <summary>
        /// Initialize object (the top most node)
        /// </summary>
        public void Initialize()
        {
            foreach (Item child in this.Children)
            {
                child.Parent = this;
                child.Initialize();
            }
        }

        /// <summary>
        /// Count the total number of difference items within
        /// </summary>
        /// <param name="collection"></param>
        /// <returns></returns>
        public int CountTotalDifferences(Item node)
        {
            int count = 0;

            if (node.Children.Count != 0)
            {
                foreach (Item item in node.Children)
                {
                    if (item.Children.Count == 0)
                    {
                        if (item._isChecked == true)
                        {
                            count++;
                        }
                    }
                    else
                    {
                        count += CountTotalDifferences(item);
                    }
                }
            }
            else
            {
                count = 1;
            }

            return count;
        }

        /// <summary>
        /// Count the number of differences that are set to IsChecked within
        /// </summary>
        /// <param name="node"></param>
        /// <returns></returns>
        public int CountTotalAccepted(Item node)
        {
            int count = 0;

            if (node.Children.Count != 0)
            {
                foreach (Item item in node.Children)
                {
                    if (item.Children.Count == 0)
                    {
                        if (item._isChecked == true)
                        {
                            count++;
                        }
                        //else
                        //{
                        //    //count--;
                        //}
                    }
                    else
                    {
                        count += CountTotalAccepted(item);
                    }
                }
            }
            else
            {
                if (_isChecked == true)
                {
                    count = 1;
                }
                else
                {
                    count = 0;
                }
            }

            return count;
        }

        /// <summary>
        /// Collection changed notification
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Children_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            if (e.NewItems != null)
            {
                foreach (Item item in e.NewItems)
                {
                    item.PropertyChanged += Item_PropertyChanged;
                }
            }
            if (e.OldItems != null)
            {
                foreach (Item item in e.OldItems)
                {
                    item.PropertyChanged -= Item_PropertyChanged;
                }
            }
        }

        /// <summary>
        /// Updates the Accepted property anytime collection has changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void Item_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            //if (e.PropertyName!="Accepted");
            FirePropertyChanged(() => Accepted);
        }
    }
}
